/*
 * UNIX RFCNB (RFC1001/RFC1002) NetBIOS implementation
 *
 * Version 1.0
 * RFCNB Defines
 *
 * Copyright (C) Richard Sharpe 1996
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SMBLIB_RFCNB_H
#define __SMBLIB_RFCNB_H

#include <stdio.h>        /* FILE */
#include <netinet/in.h>   /* in_addr */

#include "smbtypes.h"

/* Macros to extract things from the header. These are for portability
 * between architecture types where we are worried about byte order
 */
#define RFCNB_Pkt_Hdr_Len        sizeof(struct RFCNB_Hdr)  /* 4 */
#define RFCNB_Pkt_Sess_Len       72
#define RFCNB_Pkt_Retarg_Len     10
#define RFCNB_Pkt_Nack_Len       5
#define RFCNB_Pkt_Type_Offset    0
#define RFCNB_Pkt_Flags_Offset   1
#define RFCNB_Pkt_Len_Offset     2 /* Length is 2 bytes plus a flag bit */
#define RFCNB_Pkt_N1Len_Offset   4
#define RFCNB_Pkt_Called_Offset  5
#define RFCNB_Pkt_N2Len_Offset   38
#define RFCNB_Pkt_Calling_Offset 39
#define RFCNB_Pkt_Error_Offset   4
#define RFCNB_Pkt_IP_Offset      4
#define RFCNB_Pkt_Port_Offset    8

/* The next macro isolates the length of a packet, including the
 * bit in the flags
 */
#define RFCNB_Pkt_Len(p) ((unsigned)CVAL(p, 3) |       \
                         ((unsigned)CVAL(p, 2) << 8) | \
                          (((unsigned)CVAL(p, RFCNB_Pkt_Flags_Offset) & 1) << 16))

#if 0
  struct RFCNB_Hdr {   /* RFC-NetBios session header */
         BYTE  type;
         BYTE  flags;
         WORD  len;    /* ofs 2: 256*CVAL(p,2) + CVAL(p,3) + ((flags & 1) << 16). Big-endian */
       };
#endif

#define RFCNB_Pkt_Type(p) (CVAL(p, RFCNB_Pkt_Type_Offset))

#define RFCNB_Put_Pkt_Len(p,v) ((p)[1] = (((v) >> 16) & 1));   \
                               ((p)[2] = (((v) >> 8) & 0xFF)); \
                               ((p)[3] = ((v) & 0xFF));


/* Protocol defines we need
 */
#define RFCNB_SESSION_MESSAGE     0
#define RFCNB_SESSION_REQUEST     0x81
#define RFCNB_SESSION_ACK         0x82
#define RFCNB_SESSION_REJ         0x83
#define RFCNB_SESSION_RETARGET    0x84
#define RFCNB_SESSION_KEEP_ALIVE  0x85


/* Error responses
 *
 * these should follow the spec ... is there one ?
 */
#define RFCNBE_NoSpace        1   /* Could not allocate space for a struct */
#define RFCNBE_BadName        2   /* Could not translate a name            */
#define RFCNBE_BadRead        3   /* Read sys call failed                  */
#define RFCNBE_BadWrite       4   /* Write Sys call failed                 */
#define RFCNBE_ProtErr        5   /* Protocol Error                        */
#define RFCNBE_ConGone        6   /* Connection dropped                    */
#define RFCNBE_BadHandle      7   /* Handle passed was bad                 */
#define RFCNBE_BadSocket      8   /* Problems creating socket              */
#define RFCNBE_ConnectFailed  9   /* Connect failed                        */
#define RFCNBE_CallRejNLOCN  10   /* Call rejected, not listening on CN    */
#define RFCNBE_CallRejNLFCN  11   /* Call rejected, not listening for CN   */
#define RFCNBE_CallRejCNNP   12   /* Call rejected, called name not present */
#define RFCNBE_CallRejInfRes 13   /* Call rejetced, name ok, no resources   */
#define RFCNBE_CallRejUnSpec 14   /* Call rejected, unspecified error      */
#define RFCNBE_BadParam      15   /* Bad parameters passed ...             */
#define RFCNBE_Timeout       16   /* IO Timed out                          */


/*
 * Structures
 */

struct redirect_addr {
       struct in_addr        ip_addr;
       unsigned short        port;
       struct redirect_addr *next;
     };

struct RFCNB_Con {
       int    fd;             /* Socket descripter for TCP connection */
       int    last_err;       /* last error code                      */
       int    timeout;        /* milli-secs before I/O times out      */
       int    redirects;      /* How many times we were redirected    */

       struct redirect_addr *redirect_list;  /* First is first address */
       struct redirect_addr *last_addr;
     };

struct SMB_protocol_list {
       int         value;
       const char *name;
     };

extern const struct SMB_protocol_list SMB_protocols[];

#include <sys/pack_on.h>

typedef struct RFCNB_Hdr {     /* RFC-NetBios header */
        BYTE  type;
        BYTE  flags;
        WORD  len;             /* big-endian length */
      } RFCNB_Hdr;

struct RFCNB_SessionPkt {
       BYTE  N1;
       BYTE  called [33];
       BYTE  N2;
       BYTE  calling[33];
     };

struct RFCNB_NS_Hdr {        /* Only for little-endian machines */
       WORD  name_trn_id;
       WORD  flg_rcode    : 4;
       WORD  flg_nm_flags : 7;
       WORD  flg_opcode   : 3;
       WORD  flg_response : 1;
       WORD  QdCount;
       WORD  AnCount;
       WORD  NsCount;
       WORD  ArCount;
     };

#include <sys/pack_off.h>

#define DATA_MARKER  0xDEAFBABEL  /* marker inserted at 'RFCNB_Pkt::data[len]' */

struct RFCNB_Pkt {
       struct RFCNB_Pkt *next;
       BYTE             *data;    /* The data in this portion */
       int               len;     /* length of data portion */
     };

#define RFCNB_ALLOC_PKT(pkt,sz,fail_action)  do {                          \
                                               pkt = RFCNB_Alloc_Pkt (sz); \
                                               if (!pkt)                   \
                                                  fail_action;             \
                                             } while (0)

#define RFCNB_FREE_PKT(pkt)  RFCNB_Free_Pkt (pkt), pkt = NULL


SMB_EXPORT int RFCNB_errno;
SMB_EXPORT int RFCNB_saved_errno;   /* Save this from point of error */
SMB_EXPORT int RFCNB_Timeout;

typedef void (*ProtocolPrinter) (FILE *file, BOOL recv,
                                 const struct RFCNB_Pkt*,
                                 int hdr_len, int data_len);

SMB_EXPORT ProtocolPrinter Prot_Print_Routine;

SMB_EXPORT struct RFCNB_Pkt *RFCNB_Alloc_Pkt (int data_size);
SMB_EXPORT void              RFCNB_Free_Pkt  (struct RFCNB_Pkt *pkt);

SMB_EXPORT struct RFCNB_Con *RFCNB_Listen (unsigned short port);

SMB_EXPORT struct RFCNB_Con *RFCNB_Call (const char    *called_name,
                                         const char    *calling_name,
                                         const char    *called_addr,
                                         unsigned short port);

SMB_EXPORT int  RFCNB_Session_Req (struct RFCNB_Con *con,
                                   const char       *called_name,
                                   const char       *calling_name,
                                   BOOL             *redirect,
                                   struct in_addr   *dest_IP,
                                   unsigned short   *port);

SMB_EXPORT int  RFCNB_Send    (const struct RFCNB_Con *con, const struct RFCNB_Pkt *pkt, int len);
SMB_EXPORT int  RFCNB_Send2   (const struct RFCNB_Con *con, const void *buf, size_t len);
SMB_EXPORT int  RFCNB_Recv    (const struct RFCNB_Con *con, struct RFCNB_Pkt *pkt, int len);
SMB_EXPORT int  RFCNB_Put_Pkt (const struct RFCNB_Con *con, const struct RFCNB_Pkt *pkt, int len);
SMB_EXPORT int  RFCNB_Get_Pkt (const struct RFCNB_Con *con, struct RFCNB_Pkt *pkt, int len);
SMB_EXPORT int  RFCNB_Hangup  (struct RFCNB_Con *con);

SMB_EXPORT void RFCNB_Get_Error (char *buffer, int buf_len);
SMB_EXPORT int  RFCNB_Get_Last_Error (void);
SMB_EXPORT int  RFCNB_Get_Last_Errno (void);
SMB_EXPORT int  RFCNB_Get_Error_Msg (int code, char *buf, int len);

SMB_EXPORT void RFCNB_Register_Print_Routine (ProtocolPrinter fn);

SMB_EXPORT void RFCNB_AName_To_NBName (const char *aName, char *nbName, int nblen);
SMB_EXPORT void RFCNB_NBName_To_AName (const char *nbName, char *aName, int alen);

SMB_EXPORT void RFCNB_Print_Pkt (FILE*, const char*, const struct RFCNB_Pkt*, int len);
SMB_EXPORT int  RFCNB_Name_To_IP (const char *, struct in_addr *);
SMB_EXPORT int  RFCNB_IP_Connect (struct in_addr, int port);
SMB_EXPORT int  RFCNB_Set_Sock_NoDelay (const struct RFCNB_Con *con, BOOL yes);
SMB_EXPORT int  RFCNB_Set_Timeout (int seconds);

SMB_EXPORT int  RFCNB_Close (int socket);

#endif  /* __SMBLIB_RFCNB_H */
