/*
 * RFCNB Standard includes ...
 *
 *
 *  SMBlib Standard Includes
 *
 *  Copyright (C) 1996, Richard Sharpe
 *
 * One day we will conditionalize these on OS types ...
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SMBLIB_STD_DEFINES_H
#define __SMBLIB_STD_DEFINES_H

#if !defined(BUILDING_SMB_LIB)
  #error Do not include this file if not build SMB-lib.
#endif

#if defined(WIN32) || defined(_WIN32) || defined(__CYGWIN__)
  /* This should be the only place <windows.h> is included for WIN32 targets. */
  #define WIN32_LEAN_AND_MEAN
  #define SMB_WIN32
  #include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <ctype.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <tcp.h>

#if defined(DJGPP) || defined(__WATCOMC__) || defined(__HIGHC__) || defined(__DMC__) || \
    defined(__CYGWIN__) || defined(__MINGW32__)
  #include <unistd.h>
#endif

#if defined(USE_FORTIFY)
  #include "fortify.h"
#elif defined(USE_CRTDBG) && defined(_DEBUG)   /* MSVC */
  #include <crtdbg.h>
  #undef USE_FORTIFY
#endif

#ifndef min
#define min(a,b)  (((a) < (b)) ? (a) : (b))
#endif

#ifndef max
#define max(a,b)  (((a) > (b)) ? (a) : (b))
#endif

#ifndef INADDR_NONE
#define INADDR_NONE 0xFFFFFFFF
#endif

#ifndef DIM
#define DIM(x)      (sizeof(x)/sizeof(x)[0])  /* array dimension */
#endif

#ifndef ARGSUSED
#define ARGSUSED(x) ((void)x)
#endif

#ifndef offsetof
#define offsetof(type, member) ((size_t) &((type *)0)->member)
#endif

#if defined(__DJGPP__)
  extern void _w32_rundown_run (void);  /* Watt-32: misc.c */
  #define ASSERT_EXIT()  _w32_rundown_run(), raise (SIGABRT)
#elif defined(_MSC_VER)
  #define ASSERT_EXIT()  abort()        /* Using LoadLibrary("exc-abort.dll"), causes a trace-back */
#else
  #define ASSERT_EXIT()  exit (-1)
#endif

#if !defined(NDEBUG)
  #define SMB_ASSERT(x)  ((void) (!(x) ? \
                           fprintf (stderr, "%s(%u): Assertion " \
                                    "\"" #x"\" failed.\n",       \
                                    __FILE__, __LINE__),         \
                                    ASSERT_EXIT()                \
                                       : ((void)0)))
#else
  #define SMB_ASSERT(x)  ((void)0)
#endif

#if (defined(__DJGPP__) && defined(DJGPP) && (DJGPP_MINOR < 4)) || defined(__MINGW32__) || defined(_MSC_VER)
  #define strlcpy SMB_strlcpy
  extern void strlcpy (char *dst, const char *src, int len);
#endif

#if defined(SMB_WIN32) || (defined(__DJGPP__) && (DJGPP_MINOR >= 4))
  #define HAVE_SNPRINTF
#endif

#if defined(_MSC_VER)
  #pragma warning (disable:4018) /* warning C4018: '<' : signed/unsigned mismatch */
  #define snprintf _snprintf
#endif

/*
 * Put SMB-lib options here.
 *
 * Use structures and not the hideous "writing into offset" hacks.
 */
#define USE_STRUCTS

#endif /* __SMBLIB_STD_DEFINES_H */

